library ormicida.logging;

import 'dart:async';
import 'package:ormicida_core/ormicida_core.dart';

/// The singleton logger for the current isolate.
final Logger logger = new Logger._();

class Level implements Inspectable {
  static const ALL      = const Level(   0, 'all');
  static const DEBUG    = const Level( 100, 'debug');
  static const INFO     = const Level( 200, 'info');
  static const CONFIG   = const Level( 300, 'config');
  static const OK       = const Level( 400, 'ok');
  static const WARNING  = const Level( 500, 'warning');
  static const ERROR    = const Level( 600, 'error');
  static const INTERNAL = const Level( 700, 'internal');
  static const CRITICAL = const Level( 800, 'critical');
  static const OFF      = const Level(1000, 'off');
  
  final int     value;
  final String  name;
  
  factory Level.fromJson(Map json) =>
      new Level(json['value'], json['name']);
  
  const Level(this.value, this.name);
  
  void inspect(InspectableSink sink) {
    sink.addProperty('value', value);
    sink.addProperty('name', name);
  }
  
  Map toJson() => {'value': value, 'name': name};
  
  String toString() => 'Level(value: $value, name: "$name")';
}

class LogRecord implements Inspectable {
  final Level     level;
  final String    code;
  final Map       argument;
  final DateTime  date;
  final String    domain;
  
  factory LogRecord.fromJson(Map json) =>
      new LogRecord(
          json['level'] != null ? new Level.fromJson(json['level']) : null,
          json['code'], json['argument'], json['domain'], json['date']);
  
  LogRecord(this.level, this.code, this.argument, this.domain, this.date);
  
  LogRecord.now(this.level, this.code, this.argument, this.domain)
      : date = new DateTime.now();
  
  void inspect(InspectableSink sink) {
    sink.addProperty('level', level);
    sink.addProperty('code', code);
    if (argument != null)
      sink.addProperty('argument', argument);
    sink.addProperty('date', date);
    if (domain != null)
      sink.addProperty('domain', domain);
  }
  
  toJson() {
    var result = {'level': level, 'code': code, 'date': date};
    if (argument != null) result['argument'] = argument;
    if (domain != null) result['domain'] = domain;
    return result;
  }
  
  toString() {
    final buffer = new StringBuffer();
    buffer.write('Log(code:"$code"');
    if (argument != null) buffer.write(', argument:$argument');
    buffer.write(', level:$level, date:"$date"');
    if (domain != null) buffer.write(', domain:$domain');
    buffer.write(')');
    return buffer.toString();
  }
}

class Logger {
  StreamController _controller;
  
  Logger._();
  
  Stream get onRecord {
    if (_controller == null)
      _controller = new StreamController();
    return _controller.stream;
  }
  
  void clearListeners() {
    if (_controller != null) {
      _controller.close();
      _controller = null;
    }
  }
  
  void log(LogRecord log) {
    if (_controller != null)
      _controller.add(log);
  }
  
  void logWith(Level level, String code, Map argument, String domain) {
    log(new LogRecord.now(level, code, argument, domain));
  }
  
  /// Logs with the [Level.DEBUG] level.
  void debug(String code, [argument, String domain]) =>
      logWith(Level.DEBUG, code, argument, domain);
  
  /// Logs with the [Level.INFO] level.
  void info(String code, [argument, String domain]) =>
      logWith(Level.INFO, code, argument, domain);
  
  /// Logs with the [Level.CONFIG] level.
  void config(String code, [argument, String domain]) =>
      logWith(Level.CONFIG, code, argument, domain);
  
  /// Logs with the [Level.OK] level.
  void ok(String code, [argument, String domain]) =>
      logWith(Level.OK, code, argument, domain);
  
  /// Logs with the [Level.WARNING] level.
  void warn(String code, [argument, String domain]) =>
      logWith(Level.WARNING, code, argument, domain);
  
  /// Logs with the [Level.ERROR] level.
  void error(String code, [argument, String domain]) =>
      logWith(Level.ERROR, code, argument, domain);
  
  /// Logs with the [Level.INTERNAL] level.
  void internal(String code, [argument, String domain]) =>
      logWith(Level.INTERNAL, code, argument, domain);
  
  /// Logs with the [Level.CRITICAL] level.
  void critical(String code, [argument, String domain]) =>
      logWith(Level.CRITICAL, code, argument, domain);
}
